﻿local Addon = LibStub("AceAddon-3.0"):NewAddon("MacroBank", "AceEvent-3.0", "AceConsole-3.0");
MacroBank = Addon;
local L = LibStub("AceLocale-3.0"):GetLocale("MacroBank");

local defaults = {
  ["profile"] = {
    ["Macros"] = {
      -- [1] = {
        -- [1] = "Category", -- Category
        -- [2] = {
          -- { "Description", "MacroName", 3, "/wave" }, -- Macro #1  (Description, MacroName Icon, MacroBody)
          -- { "Description2", "MacroName", 5, "/lol" }, -- Macro #2  (Description, MacroName, Icon, MacroBody)
        -- }, -- List of Macros under this name
      -- }, -- Sorted MacroName order
    },
  },
};
local FontGold = "|cffffcc00";
local FontWhite = "|cffffffff";
local IconChoice = {
  ["NumLines"] = 4,
  ["NumColumns"] = 5,
};

function Addon:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("MacroBankDB", defaults, L["Default"]);
  -- self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged");
  -- self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged");
  -- self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged");
  -- self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileNewOrDelete");
  -- self.db.RegisterCallback(self, "OnNewSet", "OnProfileNewOrDelete");
  self:RegisterEvent("ADDON_LOADED");
  self.MainFrame = self:CreateMainFrame();
end

function Addon:OnEnable()
  if ( IsAddOnLoaded("Blizzard_MacroUI") ) then
    self:ADDON_LOADED(nil, "Blizzard_MacroUI");
  end
  self.ExpandedTree = {};
  self.CurrentMacro = 0;
end

-- function Addon:OnDisable()
  -- self:UnregisterEvent("ADDON_LOADED");
-- end

-- function Addon:OnProfileChanged()
  -- this is called every time your profile changes (after the change)
  -- self:OnProfileNewOrDelete();
-- end

-- function Addon:OnProfileNewOrDelete()
-- end

function Addon:ADDON_LOADED(Event, Addon)
  if ( Addon == "Blizzard_MacroUI" ) then
    self.StartingIndex, self.EndingIndex = {}, {};
    self.StartingIndex.Global = 1;
    if ( select(4, GetBuildInfo()) < 30000 ) then
      self.EndingIndex.Global = MAX_MACROS;
      self.StartingIndex.Char = MAX_MACROS + 1;
      self.EndingIndex.Char = self.StartingIndex.Char + MAX_MACROS;
    else
      self.EndingIndex.Global = MAX_ACCOUNT_MACROS;
      self.StartingIndex.Char = MAX_ACCOUNT_MACROS + 1;
      self.EndingIndex.Char = self.StartingIndex.Char + MAX_CHARACTER_MACROS;
    end
    self.IconToIndex = {};
    for Index = 1, GetNumMacroIcons() do
      self.IconToIndex[GetMacroIconInfo(Index)] = Index;
    end
    self.MainFrame:SetParent(MacroFrame);
    self.MainFrame:ClearAllPoints();
    self.MainFrame:SetPoint("TOPLEFT", MacroFrame, "TOPRIGHT", 0, -10);
    self.MainFrame:Show();
    self:DisplayMacro();
    self:UpgradeDatabase();
  end
end

function Addon:LoadMacro(ReplaceSelected)
  if ( UnitAffectingCombat("player") ) then
    self:Print(L["Can't load macros in combat"]);
    return;
  end
  local CurrentlySelected = {};
  CurrentlySelected.CategoryIndex, CurrentlySelected.MacroIndex, CurrentlySelected.Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( CurrentlySelected.MacroIndex == 0 ) then
    return;
  end
  CurrentlySelected.Name = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][2];
  CurrentlySelected.Icon = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][3];
  -- Convert to icon #.
  CurrentlySelected.Icon = self.IconToIndex[CurrentlySelected.Icon] or self.IconToIndex[string.gsub(CurrentlySelected.Icon, "%.blp", "")] or 1;
  CurrentlySelected.Body = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][4];
  local MacroName, MacroIndex;
  local NumMacros = {};
  NumMacros.Global, NumMacros.Char = GetNumMacros();
  local IsCharMacro;
  if ( MacroFrame.selectedTab == 1 ) then
    MacroSet, IsCharMacro = "Global", false;
  else
    MacroSet, IsCharMacro = "Char", true;
  end
  if ( ReplaceSelected and MacroFrame.selectedMacro ) then
    MacroIndex = MacroFrame.selectedMacro;
    CurrentlySelected.Name, CurrentlySelected.Icon, _, _ = GetMacroInfo(MacroFrame.selectedMacro);
  else
    for Index = self.StartingIndex[MacroSet], self.EndingIndex[MacroSet] do
      MacroName, _, _, _ = GetMacroInfo(Index);
      if ( MacroName == CurrentlySelected.Name ) then
        MacroIndex = Index;
        break;
      end
    end
  end
  if ( MacroIndex ) then
    EditMacro(MacroIndex, CurrentlySelected.Name, CurrentlySelected.Icon, CurrentlySelected.Body, 1, IsCharMacro);
  else
    if ( NumMacros[MacroSet] < (self.EndingIndex[MacroSet] - self.StartingIndex[MacroSet] + 1) ) then
      if ( select(4, GetBuildInfo()) < 30000 ) then
        CreateMacro(CurrentlySelected.Name, CurrentlySelected.Icon, CurrentlySelected.Body, 1, IsCharMacro);
      else
        CreateMacro(CurrentlySelected.Name, CurrentlySelected.Icon, CurrentlySelected.Body, IsCharMacro);
      end
    else
      self:Print(L["Not enough room to load macro"]);
    end
  end
  MacroFrame_Update();
end

function Addon:SaveMacro()
  if ( not MacroFrame.selectedMacro ) then
    return;
  end
  local Name, IconTexture, Body, _ = GetMacroInfo(MacroFrame.selectedMacro);
  -- local Icon = self.IconToIndex[IconTexture] or self.IconToIndex[string.gsub(IconTexture, "%.blp", "")] or 1;
  local Category;
  if ( self.CurrentMacro > 0 ) then
    local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
    Category = self.db.profile.Macros[CategoryIndex][1];
  else
    Category = L["TempCategory"];
  end
  self:CreateMacro(Category, L["TempDescription"], Name, IconTexture, Body);
end

function Addon:DeleteMacro(NoUpdateDisplay)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  table.remove(self.db.profile.Macros[CategoryIndex][2], MacroIndex);
  if ( #self.db.profile.Macros[CategoryIndex][2] == 0 ) then
    self.ExpandedTree[self.db.profile.Macros[CategoryIndex][1]] = nil;
    table.remove(self.db.profile.Macros, CategoryIndex);
  end
  self.CurrentMacro = 0;
  if ( not NoUpdateDisplay ) then
    self:DisplayMacro();
  end
end

function Addon:CreateMacro(Category, Description, Name, IconTexture, Body, NoUpdateDisplay)
  local CurrentlySelected = {};
  CurrentlySelected.CategoryIndex, CurrentlySelected.MacroIndex, CurrentlySelected.Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro > 0 and CurrentlySelected.MacroIndex > 0 ) then
    CurrentlySelected.Category = self.db.profile.Macros[CurrentlySelected.CategoryIndex][1];
    CurrentlySelected.Description = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][1];
    CurrentlySelected.Name = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][2];
    CurrentlySelected.Icon = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][3];
    CurrentlySelected.Body = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][4];
  end
  local CategoryIndex;
  for Key, Value in ipairs(self.db.profile.Macros) do
    if ( Value[1] == Category ) then
      CategoryIndex = Key;
      break;
    end
  end
  local NewMacro = {
    [1] = Description,
    [2] = Name,
    [3] = IconTexture,
    [4] = Body,
  };
  if ( CategoryIndex ) then
    table.insert(self.db.profile.Macros[CategoryIndex][2], NewMacro);
    table.sort(self.db.profile.Macros[CategoryIndex][2], function(a, b) if ( string.lower(a[1]) < string.lower(b[1]) ) then return true end end);
  else
    CategoryIndex = #self.db.profile.Macros + 1;
    self.db.profile.Macros[CategoryIndex] = {
      [1] = Category,
      [2] = {
        [1] = NewMacro,
      },
    };
    table.sort(self.db.profile.Macros, function(a, b) if ( string.lower(a[1]) < string.lower(b[1]) ) then return true end end);
  end
  if ( self.CurrentMacro > 0 and CurrentlySelected.MacroIndex > 0 ) then
    self.CurrentMacro = self:FindMacroIndex(CurrentlySelected.Category, CurrentlySelected.Description, CurrentlySelected.Name, CurrentlySelected.Icon, CurrentlySelected.Body);
  end
  if ( not NoUpdateDisplay ) then
    self:DisplayMacro();
  end
end

function Addon:ShowTooltip(...)
  if ( select("#", ...) == 0 ) then
    return;
  end
  GameTooltip:SetOwner(self.MainFrame, "ANCHOR_TOPLEFT");
  GameTooltip:SetText(FontWhite..select(1, ...));
  for i = 2, select("#", ...) do
    GameTooltip:AddLine(FontGold..select(i, ...));
  end
  GameTooltip:Show();
end

function Addon:DisplayMacro()
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    self.MainFrame.RestoreButton:Hide();
    self.MainFrame.ReplaceButton:Hide();
    self.MainFrame.DeleteButton:Hide();
    self.MainFrame.MacroCategory:Hide();
    self.MainFrame.MacroDescription:Hide();
    self.MainFrame.MacroName:Hide();
    self.MainFrame.MacroIcon:Hide();
    self.MainFrame.MacroBody:Hide();
  else
    self.MainFrame.RestoreButton:Show();
    self.MainFrame.ReplaceButton:Show();
    self.MainFrame.DeleteButton:Show();
    self.MainFrame.MacroCategory:Show();
    self.MainFrame.MacroDescription:Show();
    self.MainFrame.MacroName:Show();
    self.MainFrame.MacroIcon:Show();
    self.MainFrame.MacroBody:Show();
    self.MainFrame.MacroCategory:SetText(self.db.profile.Macros[CategoryIndex][1]);
    self.MainFrame.MacroDescription:SetText(self.db.profile.Macros[CategoryIndex][2][MacroIndex][1]);
    self.MainFrame.MacroName:SetText(self.db.profile.Macros[CategoryIndex][2][MacroIndex][2]);
    -- self.MainFrame.MacroIcon.NormalTexture:SetTexture(GetMacroIconInfo(self.db.profile.Macros[CategoryIndex][2][MacroIndex][3]))
    self.MainFrame.MacroIcon.NormalTexture:SetTexture(self.db.profile.Macros[CategoryIndex][2][MacroIndex][3])
    self.MainFrame.MacroBody.Text:SetText(self.db.profile.Macros[CategoryIndex][2][MacroIndex][4]);
  end
  self:MacroList_OnScroll();
end

function Addon:SetMacroCategory(Category)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  local Description = self.db.profile.Macros[CategoryIndex][2][MacroIndex][1];
  local Name = self.db.profile.Macros[CategoryIndex][2][MacroIndex][2];
  local Icon = self.db.profile.Macros[CategoryIndex][2][MacroIndex][3];
  local Body = self.db.profile.Macros[CategoryIndex][2][MacroIndex][4];
  self:DeleteMacro(true);
  self:CreateMacro(Category, Description, Name, Icon, Body, true);
  self:DisplayMacro();
end

function Addon:SetMacroDescription(Description)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  local Category = self.db.profile.Macros[CategoryIndex][1];
  self.db.profile.Macros[CategoryIndex][2][MacroIndex][1] = Description;
  local Name = self.db.profile.Macros[CategoryIndex][2][MacroIndex][2];
  local Icon = self.db.profile.Macros[CategoryIndex][2][MacroIndex][3];
  local Body = self.db.profile.Macros[CategoryIndex][2][MacroIndex][4];
  table.sort(self.db.profile.Macros[CategoryIndex][2], function(a, b) if ( string.lower(a[1]) < string.lower(b[1]) ) then return true end end);
  self.CurrentMacro = self:FindMacroIndex(Category, Description, Name, Icon, Body);
  self:DisplayMacro();
end

function Addon:SetMacroName(Name)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  self.db.profile.Macros[CategoryIndex][2][MacroIndex][2] = Name;
end

function Addon:SetMacroIcon(Icon)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  -- self.db.profile.Macros[CategoryIndex][2][MacroIndex][3] = Icon;
  self.db.profile.Macros[CategoryIndex][2][MacroIndex][3] = GetMacroIconInfo(Icon);
end

function Addon:SetMacroBody(Body)
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( self.CurrentMacro == 0 or MacroIndex == 0 ) then
    return;
  end
  self.db.profile.Macros[CategoryIndex][2][MacroIndex][4] = Body;
end

function Addon:GetMacroIndex(LineNum)
  local CurrentLine = 0;
  local NumMacros;
  for CategoryKey, CategoryValue in ipairs(self.db.profile.Macros) do
    -- Add 1 for the base entry, and if it's expanded, the # of macros for the entry.
    CurrentLine = CurrentLine + 1;
    if ( self.ExpandedTree[CategoryValue[1]] ) then
      NumMacros = #CategoryValue[2];
    else
      NumMacros = 0;
    end
    CurrentLine = CurrentLine + NumMacros;
    if ( LineNum <= CurrentLine ) then
      return CategoryKey, LineNum - (CurrentLine - NumMacros), self.ExpandedTree[CategoryValue[1]];
    end
  end
end

function Addon:GetNumDisplayedMacros()
  local NumEntries = 0;
  for CategoryKey, CategoryValue in ipairs(self.db.profile.Macros) do
    NumEntries = NumEntries + 1;
    if ( self.ExpandedTree[CategoryValue[1]] ) then
      NumEntries = NumEntries + #CategoryValue[2];
    end
  end
  return NumEntries;
end

function Addon:FindMacroIndex(Category, Description, Name, Icon, Body)
  local NumEntries = 0;
  for CategoryKey, CategoryValue in ipairs(self.db.profile.Macros) do
    NumEntries = NumEntries + 1;
    if ( CategoryValue[1] == Category ) then
      if ( self.ExpandedTree[CategoryValue[1]] ) then
        -- Line is expanded, iterate through the macros till we find it.
        for MacroKey, MacroValue in ipairs(CategoryValue[2]) do
          NumEntries = NumEntries + 1;
          if ( Description == MacroValue[1] and Name == MacroValue[2] and Icon == MacroValue[3] and Body == MacroValue[4] ) then
            return NumEntries;
          end
        end
      else
        return 0;  -- Line isn't expanded, but this was the name we were given.
      end
    else
      if ( self.ExpandedTree[CategoryValue[1]] ) then
        NumEntries = NumEntries + #CategoryValue[2];
      end
    end
  end
end

function Addon:FindCategoryIndex(CategoryIndex)
  local CurrentLine = 0;
  local NumMacros;
  for CategoryKey, CategoryValue in ipairs(self.db.profile.Macros) do
    -- Add 1 for the base entry, and if it's expanded, the # of macros for the entry.
    CurrentLine = CurrentLine + 1;
    if ( CategoryKey == CategoryIndex ) then
      return CurrentLine;
    else
      if ( self.ExpandedTree[CategoryValue[1]] ) then
        NumMacros = #CategoryValue[2];
      else
        NumMacros = 0;
      end
      CurrentLine = CurrentLine + NumMacros;
    end
  end
  return 0;
end

function Addon:SelectMacro(LineNum)
  self.MainFrame.MacroBody.Text:ClearFocus();
  local LineOffset = LineNum + FauxScrollFrame_GetOffset(self.MainFrame.MacroList.ScrollFrame);
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(LineOffset)
  self.CurrentMacro = LineOffset;
  if ( MacroIndex == 0 ) then
    self.ExpandedTree[self.db.profile.Macros[CategoryIndex][1]] = not self.ExpandedTree[self.db.profile.Macros[CategoryIndex][1]];
  end
  self:DisplayMacro();
end

function Addon:ExpandCollapse_OnClick(LineNum)
  self.MainFrame.MacroBody.Text:ClearFocus();
  local LineOffset = LineNum + FauxScrollFrame_GetOffset(self.MainFrame.MacroList.ScrollFrame);
  local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(LineOffset)
  local CurrentlySelected = {};
  CurrentlySelected.CategoryIndex, CurrentlySelected.MacroIndex, CurrentlySelected.Expanded = self:GetMacroIndex(self.CurrentMacro);
  if ( CurrentlySelected.MacroIndex > 0 ) then
    CurrentlySelected.Category = self.db.profile.Macros[CurrentlySelected.CategoryIndex][1];
    CurrentlySelected.Description = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][1];
    CurrentlySelected.Name = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][2];
    CurrentlySelected.Icon = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][3];
    CurrentlySelected.Body = self.db.profile.Macros[CurrentlySelected.CategoryIndex][2][CurrentlySelected.MacroIndex][4];
  end
  self.ExpandedTree[self.db.profile.Macros[CategoryIndex][1]] = not self.ExpandedTree[self.db.profile.Macros[CategoryIndex][1]];
  if ( self.CurrentMacro ~= 0 ) then  -- We had something selected (category or specific macro)
    if ( CurrentlySelected.MacroIndex > 0 ) then -- We had a specific macro selected
      self.CurrentMacro = self:FindMacroIndex(CurrentlySelected.Category, CurrentlySelected.Description, CurrentlySelected.Name, CurrentlySelected.Icon, CurrentlySelected.Body);
    else -- We had a category selected
      self.CurrentMacro = self:FindCategoryIndex(CurrentlySelected.CategoryIndex);
    end
  end
  self:DisplayMacro();
end

function Addon:MacroList_OnScroll()
  local LineNum, LineOffset;
  local NumEntries = self:GetNumDisplayedMacros();
  local NumScrollLines = 10;
  local LineHeight = 16;
  local CategoryIndex, MacroIndex, Expanded;
  FauxScrollFrame_Update(self.MainFrame.MacroList.ScrollFrame, NumEntries, NumScrollLines, LineHeight);
  for LineNum = 1, NumScrollLines do
    LineOffset = LineNum + FauxScrollFrame_GetOffset(self.MainFrame.MacroList.ScrollFrame);
    if ( LineOffset <= NumEntries) then
      CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(LineOffset);
      if ( MacroIndex == 0 ) then
        self.MainFrame.MacroList["ScrollLine"..LineNum].Text:SetText(self.db.profile.Macros[CategoryIndex][1]);
        if ( Expanded ) then
          self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
          self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down");
        else
          self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
          self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down");
        end
        self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:Show();
      else
        self.MainFrame.MacroList["ScrollLine"..LineNum].Text:SetText(self.db.profile.Macros[CategoryIndex][2][MacroIndex][1]);
        self.MainFrame.MacroList["ScrollLine"..LineNum].ExpandCollapse:Hide();
      end -- MacroIndex == 0
      if ( LineOffset == self.CurrentMacro ) then
        self.MainFrame.MacroList["ScrollLine"..LineNum].Highlight:Show();
      else
        self.MainFrame.MacroList["ScrollLine"..LineNum].Highlight:Hide();
      end
      self.MainFrame.MacroList["ScrollLine"..LineNum]:Show();
    else
      self.MainFrame.MacroList["ScrollLine"..LineNum]:Hide();
    end -- LineNum <= NumEntries
  end
end

function Addon:IconChoice_OnScroll()
  local LineNum, LineOffset;
  local NumEntries = math.ceil(GetNumMacroIcons() / IconChoice.NumColumns);
  local NumScrollLines = IconChoice.NumLines;
  local LineHeight = 36;
  FauxScrollFrame_Update(self.MainFrame.IconChoice.ScrollFrame, NumEntries, NumScrollLines, LineHeight);
  for LineNum = 1, NumScrollLines do
    LineOffset = LineNum + FauxScrollFrame_GetOffset(self.MainFrame.IconChoice.ScrollFrame);
    if ( LineOffset <= NumEntries) then
      -- If Start of each line <= Icon Choice <= End of each line   ( LineOffset * 5 - 4 <= Choice <= LineOffset * 5)
      if ( LineOffset * IconChoice.NumColumns - (IconChoice.NumColumns - 1) <= self.MainFrame.IconChoice.Choice and self.MainFrame.IconChoice.Choice <= LineOffset * IconChoice.NumColumns ) then
        self.MainFrame.IconChoice["ScrollLine"..LineNum]:HighlightIcon(self.MainFrame.IconChoice.Choice - (LineOffset - 1) * IconChoice.NumColumns);
      else
        self.MainFrame.IconChoice["ScrollLine"..LineNum]:HighlightIcon(0);
      end
      for Index = 1, IconChoice.NumColumns do
        if ( (LineOffset - 1) * IconChoice.NumColumns + Index <= GetNumMacroIcons() ) then
          self.MainFrame.IconChoice["ScrollLine"..LineNum]["Icon"..Index].NormalTexture:SetTexture(GetMacroIconInfo((LineOffset - 1) * IconChoice.NumColumns + Index));
          self.MainFrame.IconChoice["ScrollLine"..LineNum]["Icon"..Index]:Show();
        else
          self.MainFrame.IconChoice["ScrollLine"..LineNum]["Icon"..Index]:Hide();
        end
      end
      self.MainFrame.IconChoice["ScrollLine"..LineNum]:Show();
    else
      self.MainFrame.IconChoice["ScrollLine"..LineNum]:Hide();
    end
  end
end

function Addon:CreateScrollLine()
  local Frame = CreateFrame("Button");
  Frame:SetHeight(16);
  Frame:SetWidth(170);
  Frame:SetScript("OnEnter", function() self:ShowTooltip(L["MacroScrollLine"], L["MacroScrollLineDesc"]) end);
  Frame:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame:SetScript("OnClick", function() self:SelectMacro(Frame.LineNum) end);
  
  Frame.Highlight = Frame:CreateTexture(nil, "BACKGROUND");
  Frame.Highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight");
  Frame.Highlight:SetHeight(16);
  Frame.Highlight:SetWidth(170);
  Frame.Highlight:SetAllPoints(Frame);
  Frame.Highlight:SetBlendMode("ADD");
  Frame.Highlight:Hide();
  
  Frame.ExpandCollapse = CreateFrame("Button", nil, Frame);
  Frame.ExpandCollapse:SetHeight(16);
  Frame.ExpandCollapse:SetWidth(16);
  Frame.ExpandCollapse:SetPoint("TOPLEFT", Frame, "TOPLEFT");
  Frame.ExpandCollapse:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
  Frame.ExpandCollapse:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down");
  Frame.ExpandCollapse:SetScript("OnClick", function() self:ExpandCollapse_OnClick(Frame.LineNum) end);
  
  Frame.Text = Frame:CreateFontString(nil, "BACKGROUND", "ChatFontNormal");
  Frame.Text:SetPoint("TOPLEFT", Frame.ExpandCollapse, "TOPRIGHT", 3, 0);
  Frame.Text:SetHeight(16);
  Frame.Text:SetWidth(Frame:GetWidth() - Frame.ExpandCollapse:GetWidth() - 3);
  Frame.Text:SetJustifyH("LEFT");
  
  return Frame;
end

function Addon:CreateEditBox()
  local Frame = CreateFrame("EditBox");
  Frame:SetAutoFocus(false);
  Frame:EnableMouse(true);
  Frame:SetMaxLetters(200);
  Frame:SetBackdrop({
    ["bgFile"] = "Interface\\Tooltips\\UI-Tooltip-Background",
    ["edgeFile"] = "Interface\\Tooltips\\UI-Tooltip-Border",
    ["tile"] = true,
    ["insets"] = {
      ["top"] = 5,
      ["bottom"] = 5,
      ["left"] = 5,
      ["right"] = 5,
    },
    ["tileSize"] = 32,
    ["edgeSize"] = 16,
  });
  Frame.Title = Frame:CreateFontString(nil, "BACKGROUND", "GameFontNormalSmall");
  Frame.Title:SetPoint("BOTTOMLEFT", Frame, "TOPLEFT", 3, 0);
  Frame:SetBackdropColor(0, 0, 0, 0.95);
  Frame:SetScript("OnEditFocusLost", function(frame) frame:HighlightText(0, 0) end);
  Frame:SetScript("OnEditFocusGained", function(frame) frame:HighlightText() end);
  Frame:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame:SetFontObject(ChatFontNormal);
  Frame:SetTextInsets(6, 6, 6, 6);
  return Frame;
end

function Addon:CreateMainFrame()
  local Frame = CreateFrame("Frame");
  Frame:SetWidth(325);
  Frame:SetHeight(410);
  Frame:SetBackdrop({
    ["bgFile"] = "Interface\\DialogFrame\\UI-DialogBox-Background",
    ["edgeFile"] = "Interface\\DialogFrame\\UI-DialogBox-Border",
    ["tile"] = true,
    ["insets"] = {
      ["top"] = 12,
      ["bottom"] = 11,
      ["left"] = 11,
      ["right"] = 12,
    },
    ["tileSize"] = 32,
    ["edgeSize"] = 32,
  });
  Frame:EnableMouse(true);
  Frame:SetMovable(true);
  Frame:SetScript("OnMouseDown", function() Frame:StartMoving() end);
  Frame:SetScript("OnMouseUp", function() Frame:StopMovingOrSizing() end);
  Frame:SetClampedToScreen(true);
  Frame:SetClampRectInsets(250, -250, -350, 350)
  Frame:Hide();
  
  Frame.HeaderTexture = Frame:CreateTexture(nil, "ARTWORK");
  Frame.HeaderTexture:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header");
  Frame.HeaderTexture:SetWidth(256);
  Frame.HeaderTexture:SetHeight(64);
  Frame.HeaderTexture:SetPoint("TOP", Frame, "TOP", 0, 12);

  Frame.HeaderText = Frame:CreateFontString(nil, "ARTWORK", "GameFontNormal");
  Frame.HeaderText:SetPoint("TOP", Frame.HeaderTexture, "TOP", 0, -14);
  Frame.HeaderText:SetText(L["MacroBank"]);

  Frame.CreateButton = CreateFrame("Button", nil, Frame, "UIPanelButtonTemplate");
  Frame.CreateButton:SetPoint("TOPLEFT", Frame, "TOPLEFT", 20, -35);
  Frame.CreateButton:SetWidth(90);
  Frame.CreateButton:SetHeight(22);
  Frame.CreateButton:SetText(L["Create"]);
  Frame.CreateButton:SetScript("OnEnter", function() self:ShowTooltip(L["Create"], L["CreateDesc"]) end);
  Frame.CreateButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.CreateButton:SetScript("OnClick", function()
    local Category
    if ( self.CurrentMacro > 0 ) then
      local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro)
      Category = self.db.profile.Macros[CategoryIndex][1]
    else
      Category = L["TempCategory"]
    end
  self:CreateMacro(Category, L["TempDescription"], L["TempName"], GetMacroIconInfo(1), "")
  end);

  Frame.SaveButton = CreateFrame("Button", nil, Frame, "UIPanelButtonTemplate");
  Frame.SaveButton:SetPoint("TOP", Frame.CreateButton, "BOTTOM", 0, -10);
  Frame.SaveButton:SetWidth(90);
  Frame.SaveButton:SetHeight(22);
  Frame.SaveButton:SetText(L["Save"]);
  Frame.SaveButton:SetScript("OnEnter", function() self:ShowTooltip(L["Save"], L["SaveDesc"]) end);
  Frame.SaveButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.SaveButton:SetScript("OnClick", function() self:SaveMacro() end);

  Frame.RestoreButton = CreateFrame("Button", nil, Frame, "UIPanelButtonTemplate");
  Frame.RestoreButton:SetPoint("TOP", Frame.SaveButton, "BOTTOM", 0, -10);
  Frame.RestoreButton:SetWidth(90);
  Frame.RestoreButton:SetHeight(22);
  Frame.RestoreButton:SetText(L["Restore"]);
  Frame.RestoreButton:SetScript("OnEnter", function() self:ShowTooltip(L["Restore"], L["RestoreDesc"]) end);
  Frame.RestoreButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.RestoreButton:SetScript("OnClick", function() self:LoadMacro() end);

  Frame.ReplaceButton = CreateFrame("Button", nil, Frame, "UIPanelButtonTemplate");
  Frame.ReplaceButton:SetPoint("TOP", Frame.RestoreButton, "BOTTOM", 0, -10);
  Frame.ReplaceButton:SetWidth(90);
  Frame.ReplaceButton:SetHeight(22);
  Frame.ReplaceButton:SetText(L["Replace"]);
  Frame.ReplaceButton:SetScript("OnEnter", function() self:ShowTooltip(L["Replace"], L["ReplaceDesc"]) end);
  Frame.ReplaceButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.ReplaceButton:SetScript("OnClick", function() self:LoadMacro(true) end);

  Frame.DeleteButton = CreateFrame("Button", nil, Frame, "UIPanelButtonTemplate");
  Frame.DeleteButton:SetPoint("TOP", Frame.ReplaceButton, "BOTTOM", 0, -10);
  Frame.DeleteButton:SetWidth(90);
  Frame.DeleteButton:SetHeight(22);
  Frame.DeleteButton:SetText(L["Delete"]);
  Frame.DeleteButton:SetScript("OnEnter", function() self:ShowTooltip(L["Delete"], L["DeleteDesc"]) end);
  Frame.DeleteButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.DeleteButton:SetScript("OnClick", function() self:DeleteMacro() end);
  Frame.DeleteButton:Hide();

  Frame.MacroList = self:CreateMacroList();
  Frame.MacroList:SetParent(Frame);
  Frame.MacroList:SetPoint("TOPRIGHT", Frame, "TOPRIGHT", -20, -25);
  
  Frame.MacroCategory = self:CreateEditBox();
  Frame.MacroCategory:SetParent(Frame);
  Frame.MacroCategory:SetWidth(120);
  Frame.MacroCategory:SetHeight(26);
  Frame.MacroCategory:SetPoint("TOPRIGHT", Frame, "TOP", -10, -210);
  Frame.MacroCategory.Title:SetText(L["Category"]);
  Frame.MacroCategory:SetScript("OnEnter", function() self:ShowTooltip(L["Category"], L["CategoryDesc"]) end);
  Frame.MacroCategory:SetScript("OnEnterPressed", function(frame) self:SetMacroCategory(frame:GetText()) frame:ClearFocus() end);
  Frame.MacroCategory:SetScript("OnEscapePressed", function(frame) frame:ClearFocus() self:DisplayMacro() end);

  Frame.MacroDescription = self:CreateEditBox();
  Frame.MacroDescription:SetParent(Frame);
  Frame.MacroDescription:SetWidth(120);
  Frame.MacroDescription:SetHeight(26);
  Frame.MacroDescription:SetPoint("TOPLEFT", Frame, "TOP", 10, -210);
  Frame.MacroDescription.Title:SetText(L["Description"]);
  Frame.MacroDescription:SetScript("OnEnter", function() self:ShowTooltip(L["Description"], L["DescriptionDesc"]) end);
  Frame.MacroDescription:SetScript("OnEnterPressed", function(frame) self:SetMacroDescription(frame:GetText()) frame:ClearFocus() end);
  Frame.MacroDescription:SetScript("OnEscapePressed", function(frame) frame:ClearFocus() self:DisplayMacro() end);

  Frame.MacroName = self:CreateEditBox();
  Frame.MacroName:SetParent(Frame);
  Frame.MacroName:SetWidth(160);
  Frame.MacroName:SetHeight(26);
  Frame.MacroName:SetPoint("TOPLEFT", Frame.MacroCategory, "BOTTOMLEFT", 0, -15);
  Frame.MacroName.Title:SetText(L["Name"]);
  Frame.MacroName:SetScript("OnEnter", function() self:ShowTooltip(L["Name"], L["NameDesc"]) end);
  Frame.MacroName:SetScript("OnEnterPressed", function(frame) self:SetMacroName(frame:GetText()) frame:ClearFocus() end);
  Frame.MacroName:SetScript("OnEscapePressed", function(frame) frame:ClearFocus() self:DisplayMacro() end);
  Frame.MacroName:SetMaxLetters(16);
  
  Frame.MacroIcon = self:CreateIconButton();
  Frame.MacroIcon:SetParent(Frame);
  Frame.MacroIcon:SetPoint("BOTTOMLEFT", Frame.MacroName, "BOTTOMRIGHT", 10, 0);
  Frame.MacroIcon:SetScript("OnClick", function() Frame.IconChoice:Show() end);
  Frame.MacroIcon:SetScript("OnEnter", function(frame)
    frame.HighlightTexture:Show()
    self:ShowTooltip(L["Icon"], L["IconDesc"])
  end);
  Frame.MacroIcon:SetScript("OnLeave", function(frame)
    frame.HighlightTexture:Hide()
    GameTooltip:Hide()
  end);
  
  Frame.IconChoice = self:CreateIconChoiceFrame();
  Frame.IconChoice:SetParent(Frame);
  
  Frame.MacroBody = self:CreateMacroBody();
  Frame.MacroBody:SetParent(Frame);
  Frame.MacroBody:SetPoint("BOTTOM", Frame, "BOTTOM", 0, 25);
  
  return Frame;
end

function Addon:CreateMacroList()
  local Frame = CreateFrame("Frame");
  Frame:SetHeight(170);
  Frame:SetWidth(192);
  Frame:SetBackdrop({
    ["bgFile"] = "Interface\\DialogFrame\\UI-DialogBox-Background",
    ["edgeFile"] = "Interface\\Tooltips\\UI-Tooltip-Border",
    ["tile"] = true,
    ["insets"] = {
      ["top"] = 5,
      ["bottom"] = 5,
      ["left"] = 5,
      ["right"] = 5,
    },
    ["tileSize"] = 32,
    ["edgeSize"] = 16,
  });
  Frame.ScrollFrame = CreateFrame("ScrollFrame", "MacroBank_MacroListScrollFrame", Frame, "FauxScrollFrameTemplate");
  Frame.ScrollFrame:SetPoint("TOPLEFT", Frame, "TOPLEFT", 5, -5);
  Frame.ScrollFrame:SetHeight(160);
  Frame.ScrollFrame:SetWidth(160);
  if ( select(4, GetBuildInfo()) < 30000 ) then
    Frame.ScrollFrame:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(16, function() self:MacroList_OnScroll() end) end);
  else
    Frame.ScrollFrame:SetScript("OnVerticalScroll", function(frame, offset) FauxScrollFrame_OnVerticalScroll(frame, offset, 16, function() self:MacroList_OnScroll() end) end);
  end
  
  Frame.ScrollLine1 = self:CreateScrollLine();
  Frame.ScrollLine1:SetParent(Frame);
  Frame.ScrollLine1:SetPoint("TOPLEFT", Frame.ScrollFrame, "TOPLEFT", 0, 0);
  Frame.ScrollLine1.LineNum = 1;
  for Index = 2, 10 do
    Frame["ScrollLine"..Index] = self:CreateScrollLine();
    Frame["ScrollLine"..Index]:SetParent(Frame);
    Frame["ScrollLine"..Index]:SetPoint("TOPLEFT", Frame["ScrollLine"..(Index - 1)], "BOTTOMLEFT", 0, 0);
    Frame["ScrollLine"..Index].LineNum = Index;
  end
  return Frame;
end

function Addon:CreateMacroBody()
  local Frame = CreateFrame("Frame");
  Frame:SetHeight(95);
  Frame:SetWidth(303);
  Frame:SetBackdrop({
    ["bgFile"] = "Interface\\DialogFrame\\UI-DialogBox-Background",
    ["edgeFile"] = "Interface\\Tooltips\\UI-Tooltip-Border",
    ["tile"] = true,
    ["insets"] = {
      ["top"] = 5,
      ["bottom"] = 5,
      ["left"] = 5,
      ["right"] = 5,
    },
    ["tileSize"] = 32,
    ["edgeSize"] = 16,
  });
  
  Frame.ScrollFrame = CreateFrame("ScrollFrame", "MacroBank_MacroBodyScrollFrame", Frame, "UIPanelScrollFrameTemplate");
  Frame.ScrollFrame:SetHeight(85);
  Frame.ScrollFrame:SetWidth(270);
  Frame.ScrollFrame:SetPoint("TOPLEFT", Frame, "TOPLEFT", 7, -5);

  Frame.Text = CreateFrame("EditBox", nil, Frame.ScrollFrame);
  Frame.Text:SetHeight(85);
  Frame.Text:SetWidth(286);
  Frame.Text:SetAllPoints(Frame.ScrollFrame);
  Frame.Text:SetMultiLine(true);
  Frame.Text:SetAutoFocus(false);
  Frame.Text:SetMaxLetters(255);  -- Can't use MACROFRAME_CHAR_LIMIT at creation, as the MacroUI hasn't loaded.
  Frame.Text:SetFontObject(GameFontHighlightSmall);
  Frame.Text:SetScript("OnEscapePressed", function(frame) frame:ClearFocus() end);
  Frame.Text:SetScript("OnEditFocusLost", function(frame) frame:HighlightText(0, 0) self:SetMacroBody(frame:GetText()) end);
  Frame.Text:SetScript("OnEnter", function() self:ShowTooltip(L["Macro"], L["MacroDesc"]) end);
  Frame.Text:SetScript("OnLeave", function() GameTooltip:Hide() end);
  
  if ( select(4, GetBuildInfo()) < 30000 ) then
    Frame.Text:SetScript("OnTextChanged", function(frame)
      Frame.CharLimitText:SetFormattedText(MACROFRAME_CHAR_LIMIT, frame:GetNumLetters())
      local ScrollBar = getglobal(frame:GetParent():GetName().."ScrollBar")
      local Min, Max = ScrollBar:GetMinMaxValues()
      if ( Max > 0 and frame.max ~= Max ) then
        frame.max = Max
        ScrollBar:SetValue(Max)
      end
    end);
    Frame.Text:SetScript("OnCursorChanged", function() ScrollingEdit_OnCursorChanged(arg1, arg2, arg3, arg4) end);
    Frame.Text:SetScript("OnUpdate", function() ScrollingEdit_OnUpdate(Frame.ScrollFrame) end);
  else
    Frame.Text:SetScript("OnTextChanged", function(frame)
      Frame.CharLimitText:SetFormattedText(MACROFRAME_CHAR_LIMIT, frame:GetNumLetters())
      ScrollingEdit_OnTextChanged(frame, frame:GetParent())
    end);
    Frame.Text:SetScript("OnCursorChanged", function(frame, x, y, w, h) ScrollingEdit_OnCursorChanged(frame, x, y, w, h) end);
    Frame.Text:SetScript("OnUpdate", function(frame, elapsed) ScrollingEdit_OnUpdate(frame, elapsed, frame:GetParent()) end);
  end
  Frame.ScrollFrame:SetScrollChild(Frame.Text);

  Frame.FocusButton = CreateFrame("Button", nil, Frame);
  Frame.FocusButton:SetAllPoints(Frame.ScrollFrame);
  Frame.FocusButton:SetScript("OnClick", function() Frame.Text:SetFocus() end);
  Frame.FocusButton:SetScript("OnEnter", function() self:ShowTooltip(L["Macro"], L["MacroDesc"]) end);
  Frame.FocusButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  
  Frame.CharLimitText = Frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
  Frame.CharLimitText:SetPoint("TOP", Frame, "BOTTOM", 0, -3);
  
  return Frame;
end

function Addon:CreateIconButton()
  local Frame = CreateFrame("Button");
  Frame:SetHeight(36);
  Frame:SetWidth(36);
  Frame:SetScript("OnEnter", function() Frame.HighlightTexture:Show() end);
  Frame:SetScript("OnLeave", function() Frame.HighlightTexture:Hide() end);
  
  Frame.BackgroundTexture = Frame:CreateTexture(nil, "BACKGROUND");
  Frame.BackgroundTexture:SetTexture("Interface\\Buttons\\UI-EmptySlot-Disabled");
  Frame.BackgroundTexture:SetWidth(64);
  Frame.BackgroundTexture:SetHeight(64);
  Frame.BackgroundTexture:SetPoint("CENTER", Frame, "CENTER", 0, -1);

  Frame.NormalTexture = Frame:CreateTexture(nil, "ARTWORK");
  Frame.NormalTexture:SetWidth(36);
  Frame.NormalTexture:SetHeight(36);
  Frame.NormalTexture:SetPoint("CENTER", Frame, "CENTER", 0, 0);

  Frame.HighlightTexture = Frame:CreateTexture(nil, "HIGHLIGHT");
  Frame.HighlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
  Frame.HighlightTexture:SetWidth(36);
  Frame.HighlightTexture:SetHeight(36);
  Frame.HighlightTexture:SetPoint("CENTER", Frame, "CENTER", 0, 0);
  Frame.HighlightTexture:SetBlendMode("ADD");
  Frame.HighlightTexture:Hide();
  
  Frame.SelectedTexture = Frame:CreateTexture(nil, "OVERLAY");
  Frame.SelectedTexture:SetTexture("Interface\\Buttons\\CheckButtonHilight");
  Frame.SelectedTexture:SetWidth(36);
  Frame.SelectedTexture:SetHeight(36);
  Frame.SelectedTexture:SetPoint("CENTER", Frame, "CENTER", 0, 0);
  Frame.SelectedTexture:SetBlendMode("ADD");
  Frame.SelectedTexture:Hide();
  
  return Frame;
end

local function HighlightIcon(Frame, IconNum, Hide)
  Frame.SelectedIcon = IconNum;
  for Index = 1, IconChoice.NumColumns do
    if ( Index == IconNum ) then
      Frame["Icon"..Index].SelectedTexture:Show()
    else
      Frame["Icon"..Index].SelectedTexture:Hide()
    end
  end
end

function Addon:CreateIconChoiceScrollLine()
  local Frame = CreateFrame("Frame");
  local Spacing = 8;
  Frame:SetHeight(36);
  Frame:SetWidth(36 * IconChoice.NumColumns + Spacing * (IconChoice.NumColumns - 1));

  Frame.Icon1 = self:CreateIconButton();
  Frame.Icon1:SetParent(Frame);
  Frame.Icon1:SetPoint("TOPLEFT", Frame, "TOPLEFT", 0, 0);
  Frame.Icon1:SetScript("OnClick", function() Frame:GetParent():SetChoice(Frame.LineNum, 1) end);
  for Index = 2, IconChoice.NumColumns do
    Frame["Icon"..Index] = self:CreateIconButton();
    Frame["Icon"..Index]:SetParent(Frame);
    Frame["Icon"..Index]:SetPoint("TOPLEFT", Frame["Icon"..(Index - 1)], "TOPRIGHT", Spacing, 0);
    Frame["Icon"..Index]:SetScript("OnClick", function() Frame:GetParent():SetChoice(Frame.LineNum, Index) end);
  end
  
  Frame.HighlightIcon = HighlightIcon;
  Frame.SelectedIcon = 0;
  return Frame;
end

local function SetChoice(Frame, Line, Icon)
  local OldLineNum = math.ceil(Frame.Choice / IconChoice.NumColumns) - FauxScrollFrame_GetOffset(Frame.ScrollFrame);
  if ( 1 <= OldLineNum and OldLineNum <= IconChoice.NumLines ) then
    Frame["ScrollLine"..OldLineNum]:HighlightIcon(0);
  end  
  local LineOffset = Line + FauxScrollFrame_GetOffset(Frame.ScrollFrame);
  Frame.Choice = (LineOffset - 1) * IconChoice.NumColumns + Icon;
  Frame["ScrollLine"..Line]:HighlightIcon(Icon);
end

function Addon:CreateIconChoiceFrame()
  local Frame = CreateFrame("Frame");
  Frame:SetPoint("CENTER", UIParent, "CENTER");
  Frame:SetHeight(230);
  Frame:SetWidth(270);
  Frame:SetToplevel(true);
  Frame:SetFrameStrata("DIALOG");
  Frame:EnableMouse(true);
  Frame:SetBackdrop({
    ["bgFile"] = "Interface\\DialogFrame\\UI-DialogBox-Background",
    ["edgeFile"] = "Interface\\DialogFrame\\UI-DialogBox-Border",
    ["tile"] = true,
    ["insets"] = {
      ["top"] = 12,
      ["bottom"] = 11,
      ["left"] = 11,
      ["right"] = 12,
    },
    ["tileSize"] = 32,
    ["edgeSize"] = 32,
  });
  Frame:SetScript("OnShow", function()
    local CategoryIndex, MacroIndex, Expanded = self:GetMacroIndex(self.CurrentMacro)
    Frame.Choice = self.db.profile.Macros[CategoryIndex][2][MacroIndex][3]
    -- Convert to Icon#
    Frame.Choice = self.IconToIndex[Frame.Choice] or self.IconToIndex[string.gsub(Frame.Choice, "%.blp", "")] or 1
    local NumEntries = math.ceil(GetNumMacroIcons() / IconChoice.NumColumns)
    local NumScrollLines = IconChoice.NumLines
    local LineHeight = 36
    FauxScrollFrame_Update(Frame.ScrollFrame, NumEntries, NumScrollLines, LineHeight)
    local Offset = math.ceil(Frame.Choice / IconChoice.NumColumns) - 1 -- with 1044 entries, # Offsets is 0-205  (1044/5 = 208.8)
    getglobal(Frame.ScrollFrame:GetName().."ScrollBar"):SetValue(Offset * LineHeight) -- Triggers OnVerticalScroll
    -- self:IconChoice_OnScroll()
  end);
  Frame:Hide();
  
  Frame.ScrollFrame = CreateFrame("ScrollFrame", "MacroBank_IconChoiceScrollFrame", Frame, "FauxScrollFrameTemplate");
  Frame.ScrollFrame:SetPoint("TOPLEFT", Frame, "TOPLEFT", 18, -18);
  Frame.ScrollFrame:SetHeight(168);
  Frame.ScrollFrame:SetWidth(212);
  if ( select(4, GetBuildInfo()) < 30000 ) then
    Frame.ScrollFrame:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(36, function() self:IconChoice_OnScroll() end) end);
  else
    Frame.ScrollFrame:SetScript("OnVerticalScroll", function(frame, offset) FauxScrollFrame_OnVerticalScroll(frame, offset, 36, function() self:IconChoice_OnScroll() end) end);
  end
  
  Frame.ScrollLine1 = self:CreateIconChoiceScrollLine();
  Frame.ScrollLine1:SetParent(Frame);
  Frame.ScrollLine1:SetPoint("TOPLEFT", Frame.ScrollFrame, "TOPLEFT", 0, 0);
  Frame.ScrollLine1.LineNum = 1;
  for Index = 2, IconChoice.NumLines do
    Frame["ScrollLine"..Index] = self:CreateIconChoiceScrollLine();
    Frame["ScrollLine"..Index]:SetParent(Frame);
    Frame["ScrollLine"..Index]:SetPoint("TOPLEFT", Frame["ScrollLine"..(Index - 1)], "BOTTOMLEFT", 0, -7);
    Frame["ScrollLine"..Index].LineNum = Index;
  end
  
  Frame.AcceptButton = CreateFrame("Button", nil, Frame, "GameMenuButtonTemplate");
  Frame.AcceptButton:SetWidth(60);
  Frame.AcceptButton:SetHeight(21);
  Frame.AcceptButton:SetPoint("BOTTOMRIGHT", Frame, "BOTTOM", -10, 10);
  Frame.AcceptButton:SetScript("OnClick", function()
    self:SetMacroIcon(Frame.Choice)
    Frame:Hide();
    self:DisplayMacro();
  end);
  Frame.AcceptButton:SetScript("OnEnter", function() self:ShowTooltip(L["AcceptIcon"], L["AcceptIconDesc"]) end);
  Frame.AcceptButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.AcceptButton:SetText(L["AcceptIcon"]);
  
  Frame.CancelButton = CreateFrame("Button", nil, Frame, "GameMenuButtonTemplate");
  Frame.CancelButton:SetWidth(60);
  Frame.CancelButton:SetHeight(21);
  Frame.CancelButton:SetPoint("BOTTOMLEFT", Frame, "BOTTOM", 10, 10);
  Frame.CancelButton:SetScript("OnClick", function() Frame:Hide() end);
  Frame.CancelButton:SetScript("OnEnter", function() self:ShowTooltip(L["Cancel"], L["CancelDesc"]) end);
  Frame.CancelButton:SetScript("OnLeave", function() GameTooltip:Hide() end);
  Frame.CancelButton:SetText(L["Cancel"]);

  Frame.SetChoice = SetChoice;
  return Frame;
end

function Addon:UpgradeDatabase()
  if ( not MacroBankDB.DBVersion ) then -- Version 1
    if ( self.db.profile.Macros ) then
      for CategoryKey, CategoryValue in pairs(self.db.profile.Macros) do
        for MacroKey, MacroValue in pairs(CategoryValue[2]) do
          if ( MacroValue[3] ) then
            MacroValue[3] = GetMacroIconInfo(MacroValue[3]);
          end
        end
      end
    end
    MacroBankDB.DBVersion = 2;
  end
end
